/*
Copyright 2017-2018 Aron Barath

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "ncbricks.h"

static void update(struct paddle* const paddle)
{
	paddle->lrect.pos.x = LOGICAL_SCALE * paddle->x;
	paddle->lrect.pos.y = LOGICAL_SCALE * g_paddle_y;
	paddle->lrect.end.x = paddle->lrect.pos.x + (LOGICAL_SCALE * paddle->width) - 1;
	paddle->lrect.end.y = paddle->lrect.pos.y + LOGICAL_SCALE - 1;
}

void paddle_reset(struct paddle* const paddle)
{
	paddle->width = PADDLE_WIDTH;
	paddle->x = (g_board_width - PADDLE_WIDTH) / 2;
	update(paddle);
}

void paddle_show(const struct paddle* const paddle)
{
	int i;

	wmove(g_game_win, g_board_top+g_paddle_y, g_board_left+paddle->x);
	waddch(g_game_win, '<' | A_BOLD);
	for(i=2;i<paddle->width;++i) { waddch(g_game_win, '#' | A_BOLD); }
	waddch(g_game_win, '>' | A_BOLD);

	invalidate_board();
}

void paddle_hide(const struct paddle* const paddle)
{
	int i;
	char ch = ' ';
	if(g_backdef) { ch = '_'; }

	wmove(g_game_win, g_board_top+g_paddle_y, g_board_left+paddle->x);
	for(i=0;i<paddle->width;++i) { waddch(g_game_win, ch); }

	invalidate_board();
}

void paddle_move(struct paddle* const paddle, int amount)
{
	int x = paddle->x + amount;

	if(0>x)
	{
		x = 0;
	}
	else
	{
		const int max_x = g_board_width - paddle->width;

		if(max_x<=x)
		{
			x = max_x;
		}
	}

	if(x!=paddle->x)
	{
		paddle_hide(paddle);
		paddle->x = x;
		paddle_show(paddle);
		update(paddle);
	}
}

void paddle_enlarge(struct paddle* const paddle, int size)
{
	int max_x;

	if(0>size && PADDLE_MINW==paddle->width)
	{
		return;
	}
	else
	if(0<size && PADDLE_MAXW==paddle->width)
	{
		return;
	}

	paddle_hide(paddle);

	paddle->x     -= size;
	paddle->width += size*2;

	max_x = g_board_width - paddle->width;

	if(0>paddle->x)
	{
		paddle->x = 0;
	}
	else
	if(max_x<paddle->x)
	{
		paddle->x = max_x;
	}

	update(paddle);
	paddle_show(paddle);
}
