/*
Copyright 2017-2018 Aron Barath

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef VEC2_H
#define VEC2_H

struct vec2
{
	int x;
	int y;
};

struct rect2
{
	struct vec2 pos;
	struct vec2 end;
};

int vec2_length(const struct vec2* const vec);
int vec2_distance(const struct vec2* const p1, const struct vec2* const p2);
int vec2_L1norm(const struct vec2* const p1, const struct vec2* const p2);
void vec2_smul(struct vec2* const dst, const struct vec2* const vec, int scalar);
void vec2_sdiv(struct vec2* const dst, const struct vec2* const vec, int scalar);
void vec2_smuldiv(struct vec2* const dst, const struct vec2* const vec, int s_mul, int s_div);
void vec2_add(struct vec2* const dst, const struct vec2* const v1, const struct vec2* const v2);
void vec2_sub(struct vec2* const dst, const struct vec2* const v1, const struct vec2* const v2);
void vec2_random(struct vec2* const dst, const int magnitude);
int vec2_intersect(struct vec2* const point, const struct vec2* const line1s, const struct vec2* const line1e, const struct vec2* const line2s, const struct vec2* const line2e);
int vec2_intersect_vr(struct vec2* const point, const struct vec2* const line1s, const struct vec2* const line1e, const int line2sx, const int line2sy, const int line2ex, const int line2ey); /* line 1 in vector form; line 2 in raw form */
void rect2_normalize(struct rect2* const rect);
int rect2_contains(const struct rect2* const rect, const struct vec2* const point);

#endif /* VEC2_H */
